@@ -660,16 +660,14 @@ static inline void nv_vunmap(NvUPtr vaddr, NvU32 page_count)
 #if defined(NV_SMP_CALL_FUNCTION_PRESENT)
 #if (NV_SMP_CALL_FUNCTION_ARGUMENT_COUNT == 4)
 #define NV_SMP_CALL_FUNCTION(func, info)                     \
-    ({                                                       \
-        int __ret = smp_call_function(func, info, 1, 1);     \
-        __ret;                                               \
-     })
+    do {                                                     \
+        smp_call_function(func, info, 1, 1);                 \
+    } while(0)
 #elif (NV_SMP_CALL_FUNCTION_ARGUMENT_COUNT == 3)
 #define NV_SMP_CALL_FUNCTION(func, info)                     \
-    ({                                                       \
-        int __ret = smp_call_function(func, info, 1);        \
-        __ret;                                               \
-     })
+    do {                                                     \
+        smp_call_function(func, info, 1);                    \
+    } while(0)
 #else
 #error "NV_SMP_CALL_FUNCTION_ARGUMENT_COUNT value unrecognized!"
 #endif
@@ -678,25 +676,22 @@ static inline void nv_vunmap(NvUPtr vaddr, NvU32 page_count)
 #else
 // Non-SMP case:
 #define NV_SMP_CALL_FUNCTION(func, info)                     \
-    ({                                                       \
+    do {                                                     \
         func(info);                                          \
-        0;                                                   \
-     })
+    } while(0)
 #endif
 
 #if defined(NV_ON_EACH_CPU_PRESENT)
 #if (NV_ON_EACH_CPU_ARGUMENT_COUNT == 4)
 #define NV_ON_EACH_CPU(func, info)                     \
-    ({                                                 \
-        int __ret = on_each_cpu(func, info, 1, 1);     \
-        __ret;                                         \
-     })
+    do {                                               \
+        on_each_cpu(func, info, 1, 1);                 \
+     } while (0)
 #elif (NV_ON_EACH_CPU_ARGUMENT_COUNT == 3)
 #define NV_ON_EACH_CPU(func, info)                     \
-    ({                                                 \
-        int __ret = on_each_cpu(func, info, 1);        \
-        __ret;                                         \
-     })
+    do {                                               \
+        on_each_cpu(func, info, 1);                    \
+     } while(0)
 #else
 #error "NV_ON_EACH_CPU_ARGUMENT_COUNT value unrecognized!"
 #endif
@@ -705,10 +700,9 @@ static inline void nv_vunmap(NvUPtr vaddr, NvU32 page_count)
 #else
 // Non-SMP case:
 #define NV_ON_EACH_CPU(func, info)                     \
-    ({                                                 \
+    do {                                               \
         func(info);                                    \
-        0;                                             \
-     })
+     } while(0)
 #endif
 
 #if defined(NV_GET_NUM_PHYSPAGES_PRESENT)
@@ -1219,6 +1213,36 @@ static inline NvU32 nv_alloc_init_flags(int cached, int contiguous, int zeroed)
     return flags;
 }
 
+/**
+ * nv_is_dma_direct - return true if direct_dma is enabled
+ *
+ * Starting with the 5.0 kernel, SWIOTLB is merged into
+ * direct_dma, so systems without an IOMMU use direct_dma.  We
+ * need to know if this is the case, so that we can use a
+ * different check for SWIOTLB enablement.
+ */
+static inline NvBool nv_is_dma_direct(struct device *dev)
+{
+    NvBool is_direct = NV_FALSE;
+
+#if defined(NV_DMA_IS_DIRECT_PRESENT)
+    if (dma_is_direct(get_dma_ops(dev)))
+        is_direct = NV_TRUE;
+#endif
+
+    return is_direct;
+}
+
+/**
+ * nv_dma_maps_swiotlb - return NV_TRUE if swiotlb is enabled
+ *
+ * SWIOTLB creates bounce buffers for the DMA mapping layer to
+ * use if a driver asks the kernel to map a DMA buffer that is
+ * outside of the device's addressable range.  The driver does
+ * not function correctly if bounce buffers are enabled for the
+ * device.  So if SWIOTLB is enabled, we should avoid making
+ * mapping calls.
+ */
 static inline NvBool nv_dma_maps_swiotlb(struct pci_dev *dev)
 {
     NvBool swiotlb_in_use = NV_FALSE;
@@ -1283,6 +1307,15 @@ static inline NvBool nv_dma_maps_swiotlb(struct pci_dev *dev)
        * driver cannot support DMA remapping. That leads to the following
        * point: "swiotlb=force" is not supported for kernels falling in above
        * range.
+       *
+       * The other "else" case that is not shown:
+       * Starting with the 5.0 kernel, swiotlb is integrated into dma_direct,
+       * which is used when there's no IOMMU.  In these kernels, ops == NULL,
+       * swiotlb_dma_ops no longer exists, and we do not support swiotlb=force
+       * (doing so would require detecting when swiotlb=force is enabled and
+       * then returning NV_TRUE even when dma_direct is in use).  So for now,
+       * we just return NV_FALSE and in nv_compute_gfp_mask() we check for
+       * whether swiotlb could possibly be used (outside of swiotlb=force).
        */
     #else
     swiotlb_in_use = (ops->map_sg == swiotlb_map_sg);
