@@ -252,7 +252,6 @@ nv_drm_atomic_crtc_duplicate_state(struct drm_crtc *crtc)
 
     __drm_atomic_helper_crtc_duplicate_state(crtc, &nv_state->base);
 
-#if defined(NV_DRM_ATOMIC_MODESET_NONBLOCKING_COMMIT_AVAILABLE)
     if ((nv_state->nv_flip =
             nv_drm_calloc(1, sizeof(*(nv_state->nv_flip)))) == NULL) {
         __nv_drm_atomic_helper_crtc_destroy_state(crtc, &nv_state->base);
@@ -261,7 +260,6 @@ nv_drm_atomic_crtc_duplicate_state(struct drm_crtc *crtc)
     }
 
     INIT_LIST_HEAD(&nv_state->nv_flip->list_entry);
-#endif
 
     nv_drm_crtc_duplicate_req_head_modeset_config(
         &(to_nv_crtc_state(crtc->state)->req_config),
@@ -283,12 +281,10 @@ static void nv_drm_atomic_crtc_destroy_state(struct drm_crtc *crtc,
 {
     struct nv_drm_crtc_state *nv_state = to_nv_crtc_state(state);
 
-#if defined(NV_DRM_ATOMIC_MODESET_NONBLOCKING_COMMIT_AVAILABLE)
     if (nv_state->nv_flip != NULL) {
         nv_drm_free(nv_state->nv_flip);
         nv_state->nv_flip = NULL;
     }
-#endif
 
     __nv_drm_atomic_helper_crtc_destroy_state(crtc, &nv_state->base);
 
@@ -501,23 +497,24 @@ static struct drm_crtc *__nv_drm_crtc_create(struct nv_drm_device *nv_dev,
                                              unsigned int head)
 {
     struct nv_drm_crtc *nv_crtc = NULL;
+    struct nv_drm_crtc_state *nv_state = NULL;
     int ret = -ENOMEM;
 
     if ((nv_crtc = nv_drm_calloc(1, sizeof(*nv_crtc))) == NULL) {
         goto failed;
     }
 
-    nv_crtc->base.state = nv_drm_calloc(1, sizeof(*nv_crtc->base.state));
-    if (nv_crtc->base.state == NULL) {
+    nv_state = nv_drm_calloc(1, sizeof(*nv_state));
+    if (nv_state == NULL) {
         goto failed_state_alloc;
     }
+
+    nv_crtc->base.state = &nv_state->base;
     nv_crtc->base.state->crtc = &nv_crtc->base;
 
     nv_crtc->head = head;
-#if defined(NV_DRM_ATOMIC_MODESET_NONBLOCKING_COMMIT_AVAILABLE)
     INIT_LIST_HEAD(&nv_crtc->flip_list);
-    spin_lock_init(&nv_crtc->flip_lock);
-#endif
+    spin_lock_init(&nv_crtc->flip_list_lock);
 
     ret = drm_crtc_init_with_planes(nv_dev->dev,
                                     &nv_crtc->base,
@@ -542,7 +539,7 @@ static struct drm_crtc *__nv_drm_crtc_create(struct nv_drm_device *nv_dev,
     return &nv_crtc->base;
 
 failed_init_crtc:
-    nv_drm_free(nv_crtc->base.state);
+    nv_drm_free(nv_state);
 
 failed_state_alloc:
     nv_drm_free(nv_crtc);
